# -*- coding: utf-8 -*-

################################################################################
# Form generated from reading UI file 'ui_settingswindow.ui'
##
# Created by: Qt User Interface Compiler version 6.4.0
##
# WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide6.QtCore import QCoreApplication, QMetaObject, QSize, Qt
from PySide6.QtWidgets import (QComboBox, QGridLayout, QHBoxLayout, QLabel,
                               QPushButton, QVBoxLayout)


class Ui_SettingsWindow(object):
    def setupUi(self, SettingsWindow):
        if not SettingsWindow.objectName():
            SettingsWindow.setObjectName(u"SettingsWindow")
        SettingsWindow.setWindowModality(Qt.ApplicationModal)
        SettingsWindow.resize(239, 126)
        SettingsWindow.setMinimumSize(QSize(239, 126))
        SettingsWindow.setMaximumSize(QSize(239, 126))
        self.gridLayout = QGridLayout(SettingsWindow)
        self.gridLayout.setObjectName(u"gridLayout")
        self.verticalLayout_3 = QVBoxLayout()
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.verticalLayout_2 = QVBoxLayout()
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.lblDesc_Theme = QLabel(SettingsWindow)
        self.lblDesc_Theme.setObjectName(u"lblDesc_Theme")

        self.horizontalLayout.addWidget(self.lblDesc_Theme)

        self.comboxTheme = QComboBox(SettingsWindow)
        self.comboxTheme.setObjectName(u"comboxTheme")

        self.horizontalLayout.addWidget(self.comboxTheme)

        self.horizontalLayout_2.addLayout(self.horizontalLayout)

        self.verticalLayout = QVBoxLayout()
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.lblDesc_Accent = QLabel(SettingsWindow)
        self.lblDesc_Accent.setObjectName(u"lblDesc_Accent")

        self.verticalLayout.addWidget(self.lblDesc_Accent)

        self.lblColourPreview = QLabel(SettingsWindow)
        self.lblColourPreview.setObjectName(u"lblColourPreview")

        self.verticalLayout.addWidget(self.lblColourPreview)

        self.horizontalLayout_2.addLayout(self.verticalLayout)

        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.btnChangeAccent = QPushButton(SettingsWindow)
        self.btnChangeAccent.setObjectName(u"btnChangeAccent")

        self.verticalLayout_2.addWidget(self.btnChangeAccent)

        self.verticalLayout_3.addLayout(self.verticalLayout_2)

        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.btnApply = QPushButton(SettingsWindow)
        self.btnApply.setObjectName(u"btnApply")

        self.horizontalLayout_3.addWidget(self.btnApply)

        self.btnCancel = QPushButton(SettingsWindow)
        self.btnCancel.setObjectName(u"btnCancel")

        self.horizontalLayout_3.addWidget(self.btnCancel)

        self.verticalLayout_3.addLayout(self.horizontalLayout_3)

        self.gridLayout.addLayout(self.verticalLayout_3, 0, 0, 1, 1)

        self.retranslateUi(SettingsWindow)

        QMetaObject.connectSlotsByName(SettingsWindow)
    # setupUi

    def retranslateUi(self, SettingsWindow):
        SettingsWindow.setWindowTitle(QCoreApplication.translate(
            "SettingsWindow", u"Settings", None))
        self.lblDesc_Theme.setText(QCoreApplication.translate(
            "SettingsWindow", u"Theme", None))
        self.lblDesc_Accent.setText(QCoreApplication.translate(
            "SettingsWindow", u"   Accent Colour   ", None))
        self.lblColourPreview.setText("")
        self.btnChangeAccent.setText(QCoreApplication.translate(
            "SettingsWindow", u"Change Accent Colour", None))
        self.btnApply.setText(QCoreApplication.translate(
            "SettingsWindow", u"Apply", None))
        self.btnCancel.setText(QCoreApplication.translate(
            "SettingsWindow", u"Cancel", None))
    # retranslateUi



import qdarktheme
from PySide6.QtCore import QSettings
from PySide6.QtGui import QIcon, QPainter, QPixmap
from PySide6.QtWidgets import QApplication, QColorDialog, QDialog

from .ui.settingswindowUI import Ui_SettingsWindow


class AppSettings:

    # Default accent colours for each theme as per qdarktheme docs
    DEFAULTS = {"dark": "#8ab4f7", "light": "#1a73e8"}

    def __init__(self, settings: QSettings, app: QApplication, icon: QIcon | QPixmap = None):
        """Initialises an AppSettings object that will store theme information in the location provided by `settings`, will automatically change
           app theme through `app`. `icon` specifies the icon that will be used for the settings window, can be left blank."""
        self.settings = settings
        self.app = app
        self.icon = icon

        self.loadValues()

        self.updateTheme()

    def loadValues(self):
        self.currentTheme: str = self.settings.value("theme", "dark")
        self.currentAccent: str = self.settings.value(
            "accent", self.DEFAULTS.get(self.currentTheme))

        if not self.currentAccent:
            self.currentAccent = self.DEFAULTS.get(self.currentTheme)

    def updateTheme(self):
        self.app.setStyleSheet(qdarktheme.load_stylesheet(
            self.currentTheme, custom_colors={"primary": self.currentAccent}))
        self.app.setPalette(qdarktheme.load_palette(self.currentTheme))

    def openSettings(self):
        window = SettingsWindow(self, self.icon)

        if window.exec() == QDialog.DialogCode.Accepted:
            self.currentTheme = window.comboxTheme.currentText().lower()
            if window.newAccent is not None:
                self.currentAccent = window.newAccent

            self.updateTheme()

    @property
    def currentAccent(self):
        return self._currentAccent

    @currentAccent.setter
    def currentAccent(self, val):
        self._currentAccent = val
        self.settings.setValue("accent", val)
        self.settings.sync()

    @property
    def currentTheme(self):
        return self._currentTheme

    @currentTheme.setter
    def currentTheme(self, val):
        self._currentTheme = val
        self.settings.setValue("theme", val)
        self.settings.sync()


class SettingsWindow(QDialog, Ui_SettingsWindow):

    def __init__(self, master: AppSettings, icon: QIcon | QPixmap = None):
        super().__init__()
        self.master = master
        self.setupUi(self)
        self.btnApply.setDefault(True)

        if icon is not None and isinstance(icon, (QIcon, QPixmap)):
            self.setWindowIcon(icon)

        self.updatePreview(master.currentAccent)
        self.newAccent = None
        self.comboxTheme.addItems(["Dark", "Light"])
        self.comboxTheme.setCurrentText(self.master.currentTheme.title())

        self.btnChangeAccent.clicked.connect(self.changeAccent)
        self.btnApply.clicked.connect(self.accept)
        self.btnCancel.clicked.connect(self.reject)

    def updatePreview(self, colour1: str, colour2: str = None):
        if colour2 is None:
            colour2 = colour1
            self.lblColourPreview.setToolTip(colour1)
        else:
            self.lblColourPreview.setToolTip(f"{colour1} -> {colour2}")

        width = self.lblColourPreview.width()
        height = self.lblColourPreview.height()

        preview = QPixmap(width, height)
        previewLeft = QPixmap(width/2, height)
        previewRight = QPixmap(width/2, height)

        previewLeft.fill(colour1)
        previewRight.fill(colour2)

        painter = QPainter(preview)
        painter.drawPixmap(0, 0, width/2, height, previewLeft)
        painter.drawPixmap(width/2, 0, width, height, previewRight)
        painter.end()

        self.lblColourPreview.setPixmap(preview)

    def changeAccent(self):
        dialog = QColorDialog(self.master.currentAccent, self)
        dialog.setWindowTitle("Choose Accent Colour")

        if dialog.exec() == dialog.DialogCode.Accepted:
            colour = dialog.selectedColor().name()

            self.updatePreview(self.master.currentAccent, colour)
            self.newAccent = colour
